# Generated by roxygen2: do not edit by hand

S3method(summary,AnalysisObject)
export("%>%")
export(MLwrap_tutorial)
export(build_model)
export(fine_tuning)
export(plot_ale)
export(plot_calibration_curve)
export(plot_confusion_matrix)
export(plot_distribution_by_class)
export(plot_gain_curve)
export(plot_graph_nn)
export(plot_integrated_gradients)
export(plot_lift_curve)
export(plot_loss_curve)
export(plot_olden)
export(plot_pdp)
export(plot_pfi)
export(plot_pr_curve)
export(plot_residuals_distribution)
export(plot_roc_curve)
export(plot_scatter_predictions)
export(plot_scatter_residuals)
export(plot_shap)
export(plot_sobol_jansen)
export(plot_tuning_results)
export(preprocessing)
export(sensitivity_analysis)
export(table_best_hyperparameters)
export(table_evaluation_results)
export(table_h2_total)
export(table_integrated_gradients_results)
export(table_olden_results)
export(table_pairwise_interaction)
export(table_pfi_results)
export(table_shap_results)
export(table_sobol_jansen_results)
importFrom(R6,R6Class)
importFrom(magrittr,"%>%")
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,model.frame)
importFrom(stats,predict)
importFrom(stats,sd)
importFrom(stats,update)
